# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: CMS.pm,v 1.149 2001/12/13 01:51:44 btrott Exp $

package MT::App::CMS;
use strict;

use Symbol;
use File::Spec;
use POSIX qw( strftime );
use MT::Util qw( encode_html format_ts offset_time_list );
use MT::App;
@MT::App::CMS::ISA = qw( MT::App );

sub init {
    my $app = shift;
    $app->SUPER::init(@_);
    $app->{vtbl} = {
            menu => \&show_menu,
            save => \&save_object,
            view => \&edit_object,
            list => \&list_objects,
            list_entries => \&list_entries,
            save_entry => \&save_entry,
            list_blogs => \&list_blogs,
            list_cat => \&list_categories,
            save_cat => \&save_categories,
            delete_confirm => \&delete_confirm,
            delete => \&delete,
            edit_permissions => \&edit_permissions,
            save_permissions => \&save_permissions,
            rebuild => \&rebuild_pages,
            start_rebuild => \&start_rebuild_pages,
            rebuild_confirm => \&rebuild_confirm,
            send_notify => \&send_notify,
            start_upload => \&start_upload,
            upload_file => \&upload_file,
            start_upload_entry => \&start_upload_entry,
            show_upload_html => \&show_upload_html,
            logout => \&logout,
            start_recover => \&start_recover,
            recover => \&recover_password,
            bookmarklets => \&bookmarklets,
            make_bm_link => \&make_bm_link,
            view_log => \&view_log,
            reset_log => \&reset_log,
            start_import => \&start_import,
            search_replace => \&search_replace,
            start_search_replace => \&start_search_replace,
    };
    $app->{default_mode} = 'list_blogs';
    $app->{template_dir} = 'cms';
    my $mode = $app->{query}->param('__mode');
    $app->{requires_login} =
        $mode && ($mode eq 'start_recover' || $mode eq 'recover') ?
        0 : 1;
    $app;
}

sub logout {
    my $app = shift;
    $app->log("User '" . $app->{author}->name . "' logged out");
    delete $app->{author};
    $app->bake_cookie(-name => 'user', -value => '', -expires => '-1y');
    $app->build_page('login.tmpl', { logged_out => 1 });
}

sub start_recover {
    my $app = shift;
    $app->build_page('recover.tmpl');
}

sub recover_password {
    my $app = shift;
    my $q = $app->{query};
    require MT::Author;
    my $name = $q->param('name');
    my $author = MT::Author->load({ name => $name });
    $app->log("Invalid author name '$name' in password recovery attempt"),
        return $app->error("No such author with name '$name'") unless $author;
    return $app->error("Author has not set birthplace; cannot recover password")
        unless $author->hint;
    my $hint = $q->param('hint');
    $app->log("Invalid attempt to recover password (used birthplace '$hint')"),
    return $app->error("Birthplace '$hint' does not match stored birthplace " .
                       "for this author") unless $author->hint eq $hint;
    return $app->error("Author does not have email address")
        unless $author->email;
    my @pool = ('a'..'z', 0..9);
    my $pass;
    for (1..8) { $pass .= $pool[ rand @pool ] }
    $author->set_password($pass);
    $author->save;
    my %head = ( To => $author->email, From => $author->email,
                 Subject => "Password Recovery" );
    my $body = <<BODY;
You requested recovery of your Movable Type password. Your password has
been changed in the system; here is the new password:

    $pass

You should be able to log in to Movable Type using this new password. Once
you have logged in, you should change your password to something more
memorable.
BODY
    require MT::Mail;
    MT::Mail->send(\%head, $body) or
        return $app->error("Error sending mail (" . MT::Mail->errstr . "); " .
                           "please fix the problem, then try again to " .
                           "recover your password.");
    $app->build_page('recover.tmpl', { recovered => 1,
                                       email => $author->email });
}

sub is_authorized {
    my $app = shift;
    require MT::Permission;
    my $blog_id = $app->{query}->param('blog_id');
    return 1 unless $blog_id;
    return unless my $author = $app->{author};
    my $perms = $app->{perms} = MT::Permission->load({
        author_id => $author->id,
        blog_id => $blog_id });
    $perms ? 1 :
        $app->error("You are not authorized to log in to this blog.");
}

sub build_page {
    my $app = shift;
    my($page, $param) = @_;
    if (my $perms = $app->{perms}) {
        $param->{can_post} = $perms->can_post;
        $param->{can_upload} = $perms->can_upload;
        $param->{can_edit_entries} =
            $perms->can_post || $perms->can_edit_all_posts;
        $param->{can_edit_templates} = $perms->can_edit_templates;
        $param->{can_edit_authors} = $perms->can_edit_authors;
        $param->{can_edit_config} = $perms->can_edit_config;
        $param->{can_rebuild} = $perms->can_rebuild;
        $param->{can_edit_categories} = $perms->can_edit_categories;
        $param->{can_edit_notifications} = $perms->can_edit_notifications;
    }
    $app->SUPER::build_page($page, $param);
}

## Application methods

sub show_menu {
    my $app = shift;
    $app->{perms} or return $app->error("No permissions");
    $app->build_page('menu.tmpl');
}

sub bookmarklets { $_[0]->build_page('bookmarklets.tmpl') }
sub make_bm_link {
    my $app = shift;
    my %param = ( have_link => 1 );
    my @show = $app->{query}->param('show');
    $param{bm_show} = join ',', @show;
    my $height = 400;
    my %show = map { $_ => 1 } @show;
    $height += 20 if $show{allow_comments};
    $height += 50 if $show{category};
    $height += 80 if $show{excerpt};
    $height += 80 if $show{text_more};
    $param{bm_win_height} = $height;
    $app->build_page('bookmarklets.tmpl', \%param);
}

sub start_import {
    my $app = shift;
    my $blog_id = $app->{query}->param('blog_id');
    my %param;
    require MT::Category;
    my $iter = MT::Category->load_iter({ blog_id => $blog_id });
    my @data;
    while (my $cat = $iter->()) {
        push @data, { category_id => $cat->id,
                      category_label => $cat->label };
    }
    @data = sort { $a->{category_label} cmp $b->{category_label} }
            @data;
    $param{category_loop} = \@data;
    $app->build_page('import.tmpl', \%param);
}

sub view_log {
    my $app = shift;
    my $author = $app->{author};
    return $app->error("Permission denied.")
        unless $author->can_view_log;
    my @log;
    require MT::Log;
    my $iter = MT::Log->load_iter(undef, { sort => 'created_on' });
    my $i = 1;
    while (my $log = $iter->()) {
        my $row = { log_message => $log->message, log_ip => $log->ip };
        $row->{created_on_formatted} = format_ts("%Y.%m.%d %H:%M:%S",
            $log->created_on);
        $row->{is_odd} = $i++ % 2 ? 1 : 0;
        push @log, $row;
    }
    my %param = ( log_entry_loop => \@log );
    $param{reset} = $app->{query}->param('reset');
    $app->build_page('view_log.tmpl', \%param);
}

sub reset_log {
    my $app = shift;
    my $author = $app->{author};
    return $app->error("Permission denied.")
        unless $author->can_view_log;
    require MT::Log;
    MT::Log->remove_all;
    $app->redirect($app->uri . '?__mode=view_log&reset=1');
}

{
    my %API = (
            author => 'MT::Author',
            comment => 'MT::Comment',
            entry   => 'MT::Entry',
            template => 'MT::Template',
            blog => 'MT::Blog',
            notification => 'MT::Notification',
        );

    sub edit_object {
        my $app = shift;
        my %param = $_[0] ? %{ $_[0] } : ();
        my $q = $app->{query};
        my $type = $q->param('_type');
        my $blog_id = $q->param('blog_id');
        my $id = $q->param('id');
        my $perms = $app->{perms};
        return $app->error("No permissions") if !$perms && $id &&
            $type ne 'author';
        if ($id &&
            ($type eq 'blog' && !$perms->can_edit_config) ||
            ($type eq 'template' && !$perms->can_edit_templates)) {
            return $app->error("Permission denied.");
        }
        if ($type eq 'entry' && !$id && !$q->param('is_bm')) {
            return $app->error("Permission denied.")
                unless $perms->can_post;
        }
        if ($type eq 'author' && $app->{author}->id != $id) {
            return $app->error("Permission denied.");
        }
        my $class = $app->_load_driver_for($type) or return;
        my $cols = $class->column_names;
        if ($id) {
            my $obj = $class->load($id) or
                return $app->error("Load failed: " . $class->errstr);
            if ($type eq 'entry') {
                return $app->error("Permission denied.")
                    unless $perms->can_edit_all_posts ||
                          ($perms->can_post &&
                              $obj->author_id == $app->{author}->id);
            }
            for my $col (@$cols) {
                $param{$col} = defined $q->param($col) ?
                    $q->param($col) : $obj->$col();
                $param{$col} = encode_html($param{$col});
            }
            if ($type eq 'entry') {
                delete $param{'author_id'};
                $blog_id = $obj->blog_id;
                my $status = $q->param('status') || $obj->status;
                $param{"status_" .
                    ($status == MT::Entry::HOLD() ? "hold" : "release")} = 1;
                my $df = $q->param('created_on_manual') ||
                    format_ts("%Y-%m-%d %H:%M:%S", $obj->created_on);
                $param{created_on_formatted} = $df;
                my $comments = $obj->comments;
                my @c_data;
                my $i = 1;
                @$comments = sort { $b->created_on cmp $a->created_on }
                             @$comments;
                for my $c (@$comments) {
                    my $df = format_ts("%Y-%m-%d %H:%M:%S", $c->created_on);
                    my $author = $c->author || '[No author]';
                    push @c_data, { comment_id => $c->id,
                                    comment_author => $author,
                                    comment_created => $df,
                                    comment_odd => ($i++ % 2 ? 1 : 0) };
                }
                $param{comment_loop} = \@c_data;
                $param{can_send_notifications} = $perms->can_send_notifications;
            } elsif ($type eq 'template') {
                $blog_id = $obj->blog_id;
                $param{has_name} = $obj->type eq 'index' ||
                                   $obj->type eq 'custom';
                $param{has_outfile} = $obj->type eq 'index';
            } elsif ($type eq 'blog') {
                $blog_id = $obj->id;
                my $at = $obj->archive_type;
                if ($at && $at ne 'None') {
                    my @at = split /,/, $at;
                    for my $at (@at) {
                        $param{'archive_type_' . $at} = 1;
                    }
                }
                $param{'status_default_' . $obj->status_default} = 1 if
                    $obj->status_default;
                $param{'archive_type_preferred_' .
                    $obj->archive_type_preferred} = 1 if
                    $obj->archive_type_preferred;
                $param{words_in_excerpt} ||= 20;
                $param{'sort_order_comments_' . $obj->sort_order_comments} = 1;
                $param{'sort_order_posts_' . $obj->sort_order_posts} = 1;
                (my $offset = $obj->server_offset) =~ s!-!_!g;
                $param{'server_offset_' . $offset} = 1;
            }
            $param{new_object} = 0;
        } else {
            $param{new_object} = 1;
            for my $col (@$cols) {
                $param{$col} = $q->param($col);
                $param{$col} = encode_html($param{$col});
            }
            if ($type eq 'entry') {
                delete $param{'author_id'};
                require MT::Blog;
                if ($blog_id) {
                    my $blog = MT::Blog->load($blog_id);
                    my $def_status = $q->param('status') ||
                                     $blog->status_default;
                    if ($def_status) {
                        $param{"status_" .
                            ($def_status == MT::Entry::HOLD() ? "hold" :
                            "release")} = 1;
                    }
                    $param{allow_comments} = $blog->allow_comments_default
                        unless defined $q->param('allow_comments');
                }
                $param{created_on_formatted} = $q->param('created_on_manual');
                if ($q->param('is_bm')) {
                    $param{text} = sprintf qq(<a href="%s">%s</a>\n\n%s),
                        scalar $q->param('link_href'),
                        scalar $q->param('link_title'),
                        $param{text};
                    require MT::Permission;
                    my $iter = MT::Permission->load_iter({ author_id =>
                        $app->{author}->id });
                    my @data;
                    while (my $perms = $iter->()) {
                        next unless $perms->can_post;
                        my $blog = MT::Blog->load($perms->blog_id);
                        next unless $blog;
                        push @data, { blog_id => $blog->id,
                                      blog_name => $blog->name,
                                      blog_allow_comments =>
                                          $blog->allow_comments_default, };
                        $param{avail_blogs}{$blog->id} = 1;
                    }
                    $param{blog_loop} = \@data;
                }
            } elsif ($type eq 'template') {
                $param{has_name} = $q->param('type') eq 'index' ||
                                   $q->param('type') eq 'custom';
                $param{has_outfile} = $q->param('type') eq 'index';
            } elsif ($type eq 'blog') {
                $param{days_on_index} = 7;
                $param{words_in_excerpt} = 20;
                $param{sort_order_posts_descend} = 1;
                $param{sort_order_comments_ascend} = 1;
                $param{file_extension} = 'html';
                $param{convert_paras} = 1;
                $param{convert_paras_comments} = 1;
                $param{ping_weblogs} = 0;
                $param{archive_type_Monthly} = 1;
                $param{archive_type_Individual} = 1;
                $param{archive_type_preferred_Individual} = 1;
                $param{server_offset_0} = 1;
                $param{status_default_1} = 1;
            }
        }
        if ($type eq 'entry') {
            require MT::Category;
            my $iter = MT::Category->load_iter({ blog_id => $blog_id });
            my $cols = MT::Category->column_names;
            my @data;
            my $cat_id = $param{category_id};
            while (my $obj = $iter->()) {
                my $row = { };
                for my $col (@$cols) {
                    $row->{'category_' . $col} = $obj->$col();
                }
                $row->{category_is_selected} = 1
                    if $cat_id && $cat_id == $obj->id;
                push @data, $row;
            }
            @data = sort { $a->{category_label} cmp $b->{category_label} }
                    @data;
            my $top = { category_id => '',
                        category_label => 'Select' };
            $top->{category_is_selected} = 1 unless $cat_id;
            unshift @data, $top;
            $param{category_loop} = \@data;
        } elsif ($type eq 'template') {
            $param{"type_$param{type}"} = 1;
        } elsif ($type eq 'blog') {
            ## Try to get the current directory; first try to use
            ## getcwd(), in case we are running with taint enabled. If
            ## that fails (inability to open directory, for example),
            ## try using cwd(). If that fails, use nothing.
            require Cwd; my $cwd = '';
            {
                my($bad);
                local $SIG{__WARN__} = sub { $bad++ };
                eval { $cwd = Cwd::getcwd() };
                if ($bad || $@) {
                    eval { $cwd = Cwd::cwd() };
                    if ($@ && $@ !~ /Insecure \$ENV{PATH}/) {
                        die $@;
                    }
                }
            }
            if (!$param{site_path}) {
                $param{site_path} = $cwd;
            }
            if (!$param{archive_path}) {
                $param{archive_path} = File::Spec->catdir($cwd, 'archives');
            }
            if (!$param{site_url}) {
                $param{site_url} = $q->url(-base => 1) . '/';
            }
            if (!$param{archive_url}) {
                $param{archive_url} = $param{site_url} . 'archives/';
            }
        }
        for my $p ($q->param) {
            $param{$p} = $q->param($p) if $p =~ /^saved/;
        }
        if ($q->param('is_bm')) {
            my $show = $q->param('bm_show') || '';
            if ($show) {
                my @show = map "show_$_", split /,/, $show;
                @param{ @show } = (1) x @show;
            }
            if ($show =~ /category/) {
                my $iter = MT::Category->load_iter({}, { sort => 'label' });
                my @i;
                my @c_data;
                my %avail_blogs = %{ $param{avail_blogs} };
                while (my $cat = $iter->()) {
                    my $blog_id = $cat->blog_id;
                    next unless $avail_blogs{$blog_id};
                    my $label = $cat->label;
                    $label =~ s!'!\\'!g;
                    $i[$blog_id] = 0 unless defined $i[$blog_id];
                    push @c_data, {
                            category_blog_id => $blog_id,
                            category_index => $i[$blog_id]++,
                            category_id => $cat->id,
                            category_label => $label
                    };
                }
                $param{category_loop} = \@c_data;
            }
            return $app->build_page("bm_entry.tmpl", \%param);
        } else {
            return $app->build_page("edit_${type}.tmpl", \%param);
        }
    }

    sub save_object {
        my $app = shift;
        my $q = $app->{query};
        my $type = $q->param('_type');
        my $id = $q->param('id');
        my $perms = $app->{perms};
        return $app->error("No permissions")
            if !$perms && $id && $type ne 'author';
        if ($id &&
            ($type eq 'blog' && !$perms->can_edit_config) ||
            ($type eq 'template' && !$perms->can_edit_templates)) {
            return $app->error("Permission denied.");
        }
        if ($type eq 'author') {
            if ($q->param('pass') ne $q->param('pass_verify')) {
                my %param = (error => 'Passwords do not match.');
                my $qual = $id ? '' : 'author_state_';
                for my $f (qw( name email url )) {
                    $param{$qual . $f} = $q->param($f);
                }
                $param{checked_blog_ids} = { map { $_ => 1 }
                                             $q->param('add_to_blog') };
                my $meth = $id ? 'edit_object' : 'edit_permissions';
                return $app->$meth(\%param);
            }

            require MT::Author;
            my $name = $q->param('name');
            my $existing = MT::Author->load({ name => $name });
            if ($existing && (!$q->param('id') ||
                $existing->id != $q->param('id'))) {
                my %param = (error => 'An author by that name already exists.');
                my $qual = $id ? '' : 'author_state_';
                for my $f (qw( name email url )) {
                    $param{$qual . $f} = $q->param($f);
                }
                $param{checked_blog_ids} = { map { $_ => 1 }
                                             $q->param('add_to_blog') };
                my $meth = $id ? 'edit_object' : 'edit_permissions';
                return $app->$meth(\%param);
            }
        }
        my $class = $app->_load_driver_for($type) or return;
        my($obj);
        if ($id) {
            $obj = $class->load($id);
        } else {
            $obj = $class->new;
        }
        my $names = $obj->column_names;
        my %values = map { $_ => scalar $q->param($_) } @$names;
        $obj->set_values(\%values);
        if ($type eq 'author') {
            if (my $pass = $q->param('pass')) {
                $obj->set_password($pass);
            }
            ## If editing profile, update cookie; otherwise we are creating
            ## a new user and should not update cookie.
            if ($id) {
                my($remember) = (split /::/, $app->{cookies}->{user}->value)[2];
                my %arg = (
                    -name => 'user',
                    -value => join('::', $obj->name, $obj->password, $remember),
                    -path => $app->path,
                );
                $arg{-expires} = '+10y' if $remember;
                $app->bake_cookie(%arg);
            }
        } elsif ($type eq 'blog') {
            my @at = $q->param('archive_type');
            $obj->archive_type( @at ? join(',', @at) : 'None' );
            my @fields = qw( email_new_comments allow_comment_html
                             autolink_urls allow_comments_default
                             convert_paras convert_paras_comments
                             allow_anon_comments ping_weblogs );
            for my $cb (@fields) {
                $obj->$cb(0) if !defined $q->param($cb);
            }
        }
        $obj->save or
            return $app->error("Saving object failed: " . $obj->errstr);
        if ($type eq 'blog' && !$id) {
            require MT::Permission;
            my $perms = MT::Permission->new;
            $perms->author_id($app->{author}->id);
            $perms->blog_id($obj->id);
            $perms->set_full_permissions;
            $perms->save;

            my $tmpl_list;
            eval { $tmpl_list = require 'MT/default-templates.pl' };
            die "Can't find default template list; where is " .
                "'default-templates.pl'?\n" if
                $@ || !$tmpl_list || ref($tmpl_list) ne 'ARRAY' || !@$tmpl_list;
            require MT::Template;
            for my $val (@$tmpl_list) {
                my $tmpl = MT::Template->new;
                $tmpl->set_values($val);
                $tmpl->blog_id($obj->id);
                $tmpl->save or
                    return $app->error("Populating blog with default " .
                                       "templates failed: " . $app->errstr);
            }
        } elsif ($type eq 'author' && !$id) {
            my $author_id = $obj->id;
            for my $blog_id ($q->param('add_to_blog')) {
                my $pe = MT::Permission->new;
                $pe->blog_id($blog_id);
                $pe->author_id($author_id);
                $pe->can_post(1);
                $pe->save;
            }
        }
        my $blog_id = $q->param('blog_id');
        if ($type eq 'blog') {
            $blog_id = $obj->id;
        }
        if ($type eq 'author' && !$id) {
            return $app->redirect($app->uri .
                '?__mode=edit_permissions&author_id=' . $obj->id);
        } elsif ($type eq 'notification') {
            return $app->redirect($app->uri .
                '?__mode=list&_type=notification&blog_id=' . $blog_id .
                '&saved=' . $obj->email);
        } elsif ($type eq 'comment') {
            return $app->redirect($app->uri .
                '?__mode=view&_type=entry&id=' . $obj->entry_id .
                '&blog_id=' . $blog_id . '&saved_comment=1');
        } else {
            return $app->redirect($app->uri .
                '?__mode=view&_type=' . $type . '&id=' . $obj->id .
                '&blog_id=' . $blog_id . '&saved=1');
        }
    }

    sub list_objects {
        my $app = shift;
        my $q = $app->{query};
        my $type = $q->param('_type');
        my $perms = $app->{perms};
        return $app->error("No permissions")
            unless $type eq 'author' || $perms;
        if ($perms &&
            (($type eq 'blog' && !$perms->can_edit_config) ||
             ($type eq 'template' && !$perms->can_edit_templates) ||
             ($type eq 'author' && !$perms->can_edit_authors))) {
            return $app->error("Permission denied.");
        }
        my $id = $q->param('id');
        my $class = $app->_load_driver_for($type) or return;
        my $blog_id = $q->param('blog_id');
        my %param;
        my %terms;
        my $cols = $class->column_names;
        for my $name (@$cols) {
            $terms{blog_id} = $blog_id, last
                if $name eq 'blog_id';
        }
        my $iter = $class->load_iter(\%terms);
        my(@data, @index_data, @custom_data);
        my(%authors);
        while (my $obj = $iter->()) {
            my $row = $obj->column_values;
            if (my $ts = $obj->created_on) {
                $row->{created_on_formatted} = format_ts("%Y.%m.%d", $ts);
            }
            if ($type eq 'author') {
                $authors{ $obj->id } = $obj->name;
            }
            if ($type eq 'template') {
                if ($obj->type eq 'index') {
                    $row->{is_odd} = @index_data % 2 ? 0 : 1;
                    push @index_data, $row;
                } elsif ($obj->type eq 'custom') {
                    $row->{is_odd} = @custom_data % 2 ? 0 : 1;
                    push @custom_data, $row;
                } else {
                    $param{'template_' . $obj->type} = $obj->id;
                }
            } else {
                $row->{is_odd} = @data % 2 ? 0 : 1;
                push @data, $row;
            }
        }
        if ($type eq 'author') {
            for my $row (@data) {
                $row->{added_by} = $authors{$row->{created_by}}
                    if $row->{created_by};
            }
        }
        if ($type eq 'template') {
            $param{object_index_loop} = \@index_data;
            $param{object_custom_loop} = \@custom_data;
        } else {
            $param{object_loop} = \@data;
        }
        $param{object_count} = scalar @data;
        $param{saved} = $q->param('saved');
        $param{saved_deleted} = $q->param('saved_deleted');
        $app->build_page("list_${type}.tmpl", \%param);
    }

    sub delete_confirm {
        my $app = shift;
        my %param = ( type => scalar $app->{query}->param('_type') );
        my @data;
        for my $id ($app->{query}->param('id')) {
            push @data, { id => $id };
        }
        $param{id_loop} = \@data;
        $param{'type_' . $param{type}} = 1;
        $param{is_zero} = @data == 0;
        $param{is_one} = @data == 1;
        $param{is_many} = !$param{is_zero} && !$param{is_one};
        $param{thisthese} = $param{is_one} ? 'this' : 'these';
        $app->build_page('delete_confirm.tmpl', \%param);
    }

    sub delete {
        my $app = shift;
        my $q = $app->{query};
        my $type = $q->param('_type');
        my $class = $app->_load_driver_for($type) or return;
        my $entry_id;
        for my $id ($q->param('id')) {
            my $obj = $class->load($id);
            next unless $obj;
            $entry_id = $obj->entry_id if !$entry_id && $type eq 'comment';
            $obj->remove;
        }
        my $blog_id = $q->param('blog_id');
        my %types = (
            comment => "__mode=view&_type=entry&blog_id=$blog_id&id=$entry_id&",
            entry => "__mode=list_entries&blog_id=$blog_id&",
            notification => "__mode=list&_type=notification&blog_id=$blog_id&",
            template => "__mode=list&_type=template&blog_id=$blog_id&",
            blog => "",
            author => "__mode=list&_type=author&",
        );
        my $url = $app->uri . '?' . $types{$type} . 'saved_deleted=1';
        return $app->build_page('reload_opener.tmpl', { url => $url });
    }

    sub _load_driver_for {
        my $app = shift;
        my($type) = @_;
        my $class = $API{$type} or
            return $app->error("Unknown object type $type");
        eval "use $class;";
        return $app->error("Loading object driver $class failed: $@") if $@;
        $class;
    }
}

sub show_upload_html {
    my $app = shift;
    defined(my $text = $app->_process_post_upload) or return;
    $app->build_page('show_upload_html.tmpl',
        { upload_html => encode_html($text) });
}

sub start_upload_entry {
    my $app = shift;
    my $q = $app->{query};
    $q->param('_type', 'entry');
    defined(my $text = $app->_process_post_upload) or return;
    $q->param('text', $text);
    $app->edit_object;
}

sub _process_post_upload {
    my $app = shift;
    my $q = $app->{query};
    my($url, $width, $height) = map $q->param($_), qw( url width height );
    if ($q->param('popup')) {
        require MT::Template;
        my $blog_id = $q->param('blog_id');
        if (my $tmpl = MT::Template->load({ blog_id => $blog_id,
                                            type => 'popup_image' })) {
            (my $base = $q->param('fname')) =~ s!\..*?$!!;
            my $ext = '.html';
            require MT::Template::Context;
            my $ctx = MT::Template::Context->new;
            $ctx->stash('image_url', $url);
            $ctx->stash('image_width', $width);
            $ctx->stash('image_height', $height);
            my $popup = $tmpl->build($ctx);
            require MT::Blog;
            my $blog = MT::Blog->load($blog_id);
            my $base_path = $q->param('site_path') ?
                $blog->site_path : $blog->archive_path;
            my $p_file = File::Spec->catfile($base_path, $base . $ext);

            ## If the popup filename already exists, we don't want to overwrite
            ## it, because it could contain valuable data; so we'll just make
            ## sure to generate the name uniquely.
            my($i, $full_base) = (0, $base . $ext);
            while (-e $p_file) {
                $full_base = $base . ++$i . $ext;
                $p_file = File::Spec->catfile($base_path, $full_base);
            }
            my $umask = oct $app->{cfg}->UploadUmask;
            my $old = umask($umask);
            local *FH;
            open FH, ">$p_file" or
                return $app->error("Can't open popup HTML file '$p_file': $!");
            print FH $popup;
            close FH;
            umask($old);
            $url = $q->param('site_path') ?
                $blog->site_url : $blog->archive_url;
            $url .= '/' unless $url =~ m!/$!;
            $full_base =~ s!^/!!;
            $url .= $full_base;
        }
        return <<HTML;
<a href="#" onclick="window.open('$url', 'popup', 'width=$width,height=$height,scrollbars=no,resizable=no,toolbar=no,directories=no,location=no,menubar=no,status=no,left=0,top=0')">image</a>
HTML
    } elsif ($q->param('include')) {
        (my $fname = $url) =~ s!^.*/!!;
        return <<HTML;
<img alt="$fname" src="$url" width="$width" height="$height" border="0" />
HTML
    } elsif ($q->param('link')) {
        return <<HTML;
<a href="$url">upload</a>
HTML
    }
}

sub list_entries {
    my $app = shift;
    my $q = $app->{query};
    my $perms = $app->{perms} or
        return $app->error("No permissions");
    if (!$perms->can_edit_all_posts && !$perms->can_post) {
        return $app->error("Permission denied.");
    }
    my $id = $q->param('id');
    if (!$id) {
        return $app->error("Permission denied.")
            unless $perms->can_post;
    }
    my %param;
    my $blog_id = $q->param('blog_id');
    my %terms = (blog_id => $blog_id);
    if (!$perms->can_edit_all_posts) {
        $terms{author_id} = $app->{author}->id;
    }
    if ((my $filter_col = $q->param('filter')) &&
        (my $val = $q->param('filter_val'))) {
        $terms{$filter_col} = $val;
        (my $url_val = $val) =~
            s!([^a-zA-Z0-9_.-])!uc sprintf "%%%02x", ord($1)!eg;
        $param{filter_args} = "&filter=$filter_col&filter_val=$url_val";
    }
    require MT::Entry;
    require MT::Category;
    require MT::Author;
    my($iter);
    my(@c_data, %cats);
    $iter = MT::Category->load_iter({ blog_id => $blog_id });
    my $i = 0;
    while (my $cat = $iter->()) {
        my $label = $cats{ $cat->id } = $cat->label;
        $label =~ s!'!\\'!g;
        push @c_data, { category_index => $i++,
                        category_id => $cat->id,
                        category_label => $label };
    }
    @c_data = sort { $a->{category_label} cmp $b->{category_label} } @c_data;
    $param{category_loop} = \@c_data;
    my(@a_data, %authors);
    $iter = MT::Author->load_iter;
    $i = 0;
    while (my $author = $iter->()) {
        $authors{ $author->id } = $author->name;
        push @a_data, { author_index => $i++,
                        author_id => $author->id,
                        author_name => $author->name };
    }
    $param{author_loop} = \@a_data;
    my $limit = $q->param('limit') || 20;
    $param{"limit_" . $limit} = 1;
    my $start_val = $q->param('start_val');
    $iter = MT::Entry->load_iter(\%terms, {
            sort => 'created_on',
            direction => 'descend',
            ($limit eq 'none' ? () : (limit => $limit)),
            ($start_val ? (start_val => $start_val) : ()),
        });
    my @data;
    $i = 1;
    while (my $obj = $iter->()) {
        my $row = $obj->column_values;
        $row->{created_on_formatted} = format_ts("%Y.%m.%d", $obj->created_on);
        $row->{author_name} = $authors{ $obj->author_id };
        $row->{category_name} = $obj->category_id ?
            $cats{ $obj->category_id } : '';
        $row->{title_short} = $obj->title;
        unless ($row->{title_short}) {
            (my $title = $obj->text) =~ s!<[^>]+>!!g;
            $row->{title_short} = substr($title, 0, 22) . '...';
        }
        $row->{title_short} = substr($row->{title_short}, 0, 22) . '...'
            if length($row->{title_short}) > 25;
        $row->{title_short} = encode_html($row->{title_short});
        $row->{status_text} = $obj->status == 1 ?
            'hold' : 'release';
        $row->{entry_odd} = $i++ % 2 ? 1 : 0;
	$row->{has_edit_access} = $perms->can_edit_all_posts ||
            $obj->author_id == $app->{author}->id;
        push @data, $row;
    }
    $param{object_loop} = \@data;
    $param{saved_deleted} = $q->param('saved_deleted');
    $app->build_page("list_entry.tmpl", \%param);
}

sub save_entry {
    my $app = shift;
    my $q = $app->{query};
    if ($q->param('preview_entry')) {
        return $app->preview_entry;
    } elsif ($q->param('reedit')) {
        $q->param('_type', 'entry');
        return $app->edit_object;
    }
    my $perms = $app->{perms} or
        return $app->error("No permissions");
    my $id = $q->param('id');
    if (!$id) {
        return $app->error("Permission denied.")
            unless $perms->can_post;
    }
    require MT::Entry;
    my($obj);
    if ($id) {
        $obj = MT::Entry->load($id);
        return $app->error("Permission denied.")
            unless $perms->can_edit_all_posts ||
                  ($perms->can_post &&
                          $obj->author_id == $app->{author}->id);
    } else {
        $obj = MT::Entry->new;
    }
    my $names = $obj->column_names;
    my %values = map { $_ => scalar $q->param($_) } @$names;
    $obj->set_values(\%values);
    $obj->allow_comments(0) if !defined $q->param('allow_comments');
    my $co = $q->param('created_on_manual');
    if ($co && $co ne $q->param('created_on_old')) {
        require Time::Local;
        unless ($co =~
            m!(\d{4})-(\d{2})-(\d{2})\s+(\d{2}):(\d{2})(?::(\d{2}))?!) {
            return $app->error("Invalid date '$co'; authored on dates must " .
                               "be in the format YYYY-MM-DD HH:MM:SS.");
        }
        my $s = $6 || 0;
        my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $1, $2, $3, $4, $5, $s;
        $obj->created_on($ts);
    }
    $obj->save or
        return $app->error("Saving entry failed: " . $obj->errstr);

    ## If the saved status is RELEASE, or if the *previous* status was
    ## RELEASE, then rebuild entry archives, indexes, and send the
    ## XML-RPC ping(s). Otherwise the status was and is HOLD, and we
    ## don't have to do anything.
    if ($obj->status == MT::Entry::RELEASE() ||
        $q->param('status_old') eq MT::Entry::RELEASE()) {
        $app->rebuild_entry( Entry => $obj ) or
            return $app->error("Your post was saved, but the rebuild failed: " .
                               $app->errstr);
        ## Need to rebuild previous and next entries, as well, in case they
        ## have MTEntryPrevious and MTEntryNext tags
        if (my $prev = $obj->previous) {
            $app->rebuild_entry( Entry => $prev ) or
                return $app->error("Your post was saved, but the rebuild " .
                                   "failed: " . $app->errstr);
        }
        if (my $next = $obj->next) {
            $app->rebuild_entry( Entry => $next ) or
                return $app->error("Your post was saved, but the rebuild " .
                                   "failed: " . $app->errstr);
        }
        $app->rebuild_indexes( BlogID => $obj->blog_id ) or
            return $app->error("Your post was saved, but the rebuild of the " .
                               "indexes failed: " . $app->errstr);
        ## Send XML-RPC ping to weblogs.com, if configured to do so
        require MT::Blog;
        my $blog = MT::Blog->load($obj->blog_id);
        if ($blog->ping_weblogs) {
            require MT::XMLRPC;
            my $res = MT::XMLRPC->weblogs_ping($blog) or
                return $app->error("Your post was saved, but the weblogs.com " .
                                   "ping failed: " . MT::XMLRPC->errstr);
        }
	if ($blog->mt_update_key) {
            require MT::XMLRPC;
	    my $res = MT::XMLRPC->mt_ping($blog) or
		return $app->error("Your post was saved, but the MT " .
				   "ping failed: " . MT::XMLRPC->errstr);
	}
    }
    if ($q->param('is_bm')) {
        require MT::Blog;
        my $blog = MT::Blog->load($obj->blog_id);
        my %param = ( blog_id => $blog->id,
                      blog_name => $blog->name,
                      blog_url => $blog->site_url,
                      entry_id => $obj->id,
                      status_released => $obj->status == MT::Entry::RELEASE() );
        $app->build_page("bm_posted.tmpl", \%param);
    } else {
        $app->redirect($app->uri . '?__mode=view&_type=entry&blog_id=' .
            $obj->blog_id . '&id=' . $obj->id .
            ($id ? '&saved_changes=1' : '&saved_added=1'));
    }
}

sub list_categories {
    my $app = shift;
    my $q = $app->{query};
    my $perms = $app->{perms} or
        return $app->error("No permissions");
    return $app->error("Permission denied.")
        unless $perms->can_edit_categories;
    require MT::Category;
    require MT::Entry;
    require MT::Author;
    my %param;
    my %authors;
    my @data;
    my $iter = MT::Category->load_iter({ blog_id => $q->param('blog_id') });
    my $cols = MT::Category->column_names;
    while (my $obj = $iter->()) {
        my $row = { };
        for my $col (@$cols) {
            $row->{'category_' . $col} = $obj->$col();
        }
        $row->{category_label} = encode_html($row->{category_label});
        $row->{category_entrycount} = MT::Entry->count({
            category_id => $obj->id });
        my $aid = $obj->author_id;
        $authors{$aid} ||= MT::Author->load($aid);
        $row->{category_author} = $authors{$aid}->name;
        push @data, $row;
    }
    @data = sort { $a->{category_label} cmp $b->{category_label} } @data;
    for (1..5) {
        push @data, { category_id => 'new', category_label => '' };
    }
    $param{category_loop} = \@data;
    $param{saved} = $q->param('saved');
    $app->build_page('edit_categories.tmpl', \%param);
}

sub save_categories {
    my $app = shift;
    my $q = $app->{query};
    my $perms = $app->{perms} or
        return $app->error("No permissions");
    return $app->error("Permission denied.")
        unless $perms->can_edit_categories;
    require MT::Category;
    my @new = $q->param('category-new');
    my $blog_id = $q->param('blog_id');
    for my $label (@new) {
        next unless $label;
        my $cat = MT::Category->new;
        $cat->blog_id($blog_id);
        $cat->label($label);
        $cat->author_id($app->{author}->id);
        $cat->save or
            return $app->error("Error on save: " . $cat->errstr);
    }
    for my $p ($q->param) {
        my($id) = $p =~ /^category-(\d+)/;
        next unless $id;
        my $cat = MT::Category->load($id) or
            return $app->error("Unknown category ID '$id'");
        $cat->label( $q->param($p) );
        $cat->save or
            return $app->error("Error on save: " . $cat->errstr);
    }
    $app->redirect($app->uri . '?__mode=list_cat&blog_id=' . $blog_id .
                   '&saved=1');
}

sub list_blogs {
    my $app = shift;
    my $q = $app->{query};
    require MT::Blog;
    require MT::Permission;
    require MT::Entry;
    require MT::Comment;
    my $author = $app->{author};
    my @perms = MT::Permission->load({ author_id => $author->id });
    my %perms = map { $_->blog_id => $_ } @perms;
    my @blogs = MT::Blog->load;
    my @data;
    my %param;
    my $i = 1;
    for my $blog (@blogs) {
        my $blog_id = $blog->id;
        my $perms = $perms{ $blog_id };
        next unless $perms && $perms->role_mask;
        $param{can_edit_authors} = 1 if $perms->can_edit_authors;
        my $row = { id => $blog->id, name => uc($blog->name),
                    description => $blog->description };
        $row->{is_odd} = $i++ % 2 ? 1 : 0;
        $row->{num_entries} = MT::Entry->count({ blog_id => $blog_id });
        $row->{num_comments} = MT::Comment->count({ blog_id => $blog_id });
        $row->{num_authors} = MT::Permission->count({ blog_id => $blog_id });
        $row->{can_delete} = $perms->can_edit_config;
        push @data, $row;
    }
    $param{blog_loop} = \@data;
    $param{can_create_blog} = $author->can_create_blog;
    $param{can_view_log} = $author->can_view_log;
    $param{saved_deleted} = $q->param('saved_deleted');
    $app->build_page('list_blog.tmpl', \%param);
}

sub preview_entry {
    my $app = shift;
    my $q = $app->{query};
    require MT::Entry;
    require MT::Builder;
    require MT::Template::Context;
    require MT::Blog;
    my $blog_id = $q->param('blog_id');
    my $blog = MT::Blog->load($blog_id);
    my $entry = MT::Entry->new;
    $entry->title($q->param('title'));
    $entry->text($q->param('text'));
    $entry->text_more($q->param('text_more'));
    my $ctx = MT::Template::Context->new;
    $ctx->stash('entry', $entry);
    $ctx->stash('blog', $blog);
    my $build = MT::Builder->new;
    my $preview_code = <<'HTML';
<p><b><$MTEntryTitle$></b></p>
<$MTEntryBody$>
<$MTEntryMore$>
HTML
    my $tokens = $build->compile($ctx, $preview_code) or
        return $app->error("Compile error: " . $build->errstr);
    defined(my $html = $build->build($ctx, $tokens)) or
        return $app->error("Build error: " . $build->errstr);
    my %param = ( preview_body => $html );
    if (my $id = $q->param('id')) {
        $param{id} = $id;
    }
    $param{new_object} = $param{id} ? 0 : 1;
    my $cols = MT::Entry->column_names;
    my @data = ({ data_name => 'author_id', data_value => $app->{author}->id });
    for my $col (@$cols) {
        next if $col eq 'created_on' || $col eq 'created_by' ||
                $col eq 'modified_on' || $col eq 'modified_by';
        push @data, { data_name => $col,
            data_value => encode_html(scalar $q->param($col)) };
    }
    for my $date (qw( created_on_old created_on_manual )) {
        push @data, { data_name => $date,
                      data_value => scalar $q->param($date) };
    }
    $param{entry_loop} = \@data;
    $app->build_page('preview_entry.tmpl', \%param);
}

sub rebuild_confirm {
    my $app = shift;
    my $blog_id = $app->{query}->param('blog_id');
    require MT::Blog;
    my $blog = MT::Blog->load($blog_id);
    my $at = $blog->archive_type || '';
    my(@at, @data);
    if ($at && $at ne 'None') {
        @at = split /,/, $at;
        @data = map { { archive_type => $_ } } @at;
    }
    my $order = join ',', @at, 'index';
    $app->build_page('rebuild_confirm.tmpl', { archive_type_loop => \@data,
                                               build_order => $order,
                                               build_next => 0 });
}

sub start_rebuild_pages {
    my $app = shift;
    my $q = $app->{query};
    my $type = $q->param('type');
    my $next = $q->param('next');
    my @order = split /,/, $type;
    my $type_name = $order[$next];
    $app->build_page('rebuilding.tmpl', {
                    build_type => $type,
                    build_next => $next,
                    build_type_name => $type_name, });
}

sub rebuild_pages {
    my $app = shift;
    my $perms = $app->{perms} or
        return $app->error("No permissions");
    return $app->error("Permission denied.")
        unless $perms->can_rebuild;
    require MT::Entry;
    require MT::Blog;
    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');
    my $order = $q->param('type');
    my @order = split /,/, $order;
    my $next = $q->param('next');
    my $done = 0;
    my $type = $order[$next];
    $next++;
    $done++ if $next >= @order;
    if ($type eq 'all') {
        $app->rebuild( BlogID => $blog_id )
            or return;
    } elsif ($type eq 'index') {
        $app->rebuild_indexes( BlogID => $blog_id ) or return;
    } elsif ($type) {
        $app->rebuild( BlogID => $blog_id,
                       ArchiveType => $type,
                       NoIndexes => 1 )
            or return;
    }
    unless ($done) {
        my $type_name = $order[$next];
        $app->build_page('rebuilding.tmpl', {
                    build_type => $order,
                    build_next => $next,
                    build_type_name => $type_name, });
    } else {
        my $all = $order =~ /,/;
        my $type = $order;
        $app->build_page('rebuilt.tmpl', { all => $all, type => $type });
    }
}

sub edit_permissions {
    my $app = shift;
    my %param = $_[0] ? %{ $_[0] } : ();
    my $q = $app->{query};
    my $author = $app->{author};
    require MT::Permission;
    require MT::Blog;
    require MT::Author;
    my @auth_perms = MT::Permission->load({ author_id => $author->id });
    ## @auth_perms is a list of Permissions for this user. List the
    ## blogs for which user has can_edit_authors permissions.
    my $all_perms = MT::Permission->perms;
    my $author_id = $q->param('author_id');
    $param{edit_author_id} = $author_id;
    my $iter = MT::Author->load_iter;
    my @a_data;
    while (my $author = $iter->()) {
        push @a_data, { author_id => $author->id,
                        author_name => $author->name };
        if ($author_id && $author->id == $author_id) {
            $a_data[-1]{author_selected} = 1;
            $param{selected_author_name} = $author->name;
            $param{can_create_blog} = $author->can_create_blog;
            $param{can_view_log} = $author->can_view_log;
        }
    }
    $param{author_loop} = \@a_data;
    my(@data, @o_data);
    for my $perms (@auth_perms) {
        next unless $perms->can_edit_authors;
        my $blog = MT::Blog->load($perms->blog_id);
        my $row = { blog_name => $blog->name, blog_id => $blog->id,
                    author_id => $author_id };
        if ($param{checked_blog_ids} && $param{checked_blog_ids}{$blog->id}) {
            $row->{is_checked} = 1;
        }
        my $p = MT::Permission->load({ blog_id => $blog->id,
                                       author_id => $author_id });
        if ($p && $p->role_mask) {
            my @p_data;
            for my $ref (@$all_perms) {
                my $meth = 'can_' . $ref->[1];
                push @p_data, { have_access => $p->$meth(),
                                prompt => $ref->[2],
                                blog_id => $blog->id,
                                author_id => $author_id,
                                mask => $ref->[0] };
            }
            $row->{perm_loop} = \@p_data;
            push @data, $row;
        } else {
            push @o_data, $row;
        }
    }
    $param{blog_loop} = \@data;
    $param{blog_no_access_loop} = \@o_data;
    $param{saved_add_to_blog} = $q->param('saved_add_to_blog');
    $param{saved} = $q->param('saved');
    $app->build_page('edit_permissions.tmpl', \%param);
}

sub save_permissions {
    my $app = shift;
    my $q = $app->{query};
    require MT::Author;
    my $my_author_id = $app->{author}->id;
    my $author_id = $q->param('author_id');
    my $author = MT::Author->load($author_id);
    $author->can_create_blog($q->param('can_create_blog') ? 1 : 0);
    $author->can_view_log($q->param('can_view_log') ? 1 : 0);
    $author->save;
    for my $p ($q->param) {
        if ($p =~ /^role_mask-(\d+)$/) {
            my($blog_id) = ($1);
            my $perms = MT::Permission->load({ author_id => $my_author_id,
                                               blog_id => $blog_id });
            return $app->error("Permission denied.")
                unless $perms && $perms->can_edit_authors;
            my $pe = $my_author_id == $author_id ? $perms :
                MT::Permission->load({ author_id => $author_id,
                                       blog_id => $blog_id });
            if (!$pe) {
                $pe = MT::Permission->new;
                $pe->blog_id($blog_id);
                $pe->author_id($author_id);
            }
            my $mask = 0;
            for my $val ($q->param($p)) {
                $mask += $val;
            }
            $pe->role_mask($mask);
            $pe->save;
        }
    }
    my $url = $app->uri . '?__mode=edit_permissions&author_id=' . $author_id .
              '&saved=1';
    if (my $blog_id = $q->param('add_role_mask')) {
        my $pe = MT::Permission->load({ author_id => $author_id,
                                        blog_id => $blog_id });
        if (!$pe) {
            $pe = MT::Permission->new;
            $pe->blog_id($blog_id);
            $pe->author_id($author_id);
        }
        $pe->can_post(1);
        $pe->save;
        require MT::Blog;
        my $blog = MT::Blog->load($blog_id);
        $url .= '&saved_add_to_blog=' . $blog->name;
    }
    $app->redirect($url);
}

sub send_notify {
    my $app = shift;
    my $q = $app->{query};
    my $entry_id = $q->param('entry_id') or
        return $app->error("No entry ID provided");
    require MT::Entry;
    require MT::Blog;
    require MT::Util;
    my $entry = MT::Entry->load($entry_id) or
        return $app->error("No such entry '$entry_id'");
    my $blog = MT::Blog->load($entry->blog_id);
    my $author = $entry->author;
    return $app->error("No email address for author '" . $author->name . "'")
        unless $author->email;
    my $body = '';
    my $cols = 72;
    my $name = $blog->name . ' Update';
    my $fill_len = $cols - length($name) - 2;
    $fill_len++ if $fill_len % 2;
    $name = ('(' x ($fill_len/2)) . ' ' . $name . ' ' . (')' x ($fill_len/2));
    $body .= $name . "\n\n";
    my $date = strftime "%B %e, %Y", offset_time_list(time, $blog);
    my $fill_left = ' ' x int(($cols - length($date)) / 2);
    $body .= "$fill_left$date\n\n\n";
    $body .= ('-' x $cols) . "\n\n";
    require Text::Wrap;
    $Text::Wrap::columns = $cols;
    if ($q->param('send_excerpt')) {
        my $excerpt = $entry->excerpt ||
          MT::Util::first_n_words($entry->text, $blog->words_in_excerpt || 20);
        local $Text::Wrap::columns = $cols - 4;
        $body .= Text::Wrap::wrap("    ", "    ", $excerpt) . "\n\n";
        $body .= ('-' x $cols) . "\n\n";
    }
    my $link = $blog->archive_url;
    $link .= '/' unless $link =~ m!/$!;
    $link .= $entry->archive_file . '#' . sprintf "%06d", $entry->id;
    $body .= "$link\n\n";
    $body .= Text::Wrap::wrap('', '', $q->param('message')) . "\n\n";
    if ($q->param('send_body')) {
        $body .= ('-' x $cols) . "\n\n";
        $body .= Text::Wrap::wrap('', '', $entry->text) . "\n";
    }
    my %head = ( To => $author->email, From => $author->email,
                 Subject => $blog->name . ' Update' );
    require MT::Notification;
    my $iter = MT::Notification->load_iter({ blog_id => $blog->id });
    my $i = 1;
    require MT::Mail;
    while (my $note = $iter->()) {
        next unless $note->email;
        push @{ $head{Bcc} }, $note->email;
        if ($i++ % 20 == 0) {
            MT::Mail->send(\%head, $body) or
                return $app->error("Error sending mail (" . MT::Mail->errstr .
                                   "); try another MailTransfer setting?");
            @{ $head{Bcc} } = ();
        }
    }
    if ($head{Bcc} && @{ $head{Bcc} }) {
        MT::Mail->send(\%head, $body) or
            return $app->error("Error sending mail (" . MT::Mail->errstr .
                               "); try another MailTransfer setting?");
    }
    $app->redirect($app->uri . '?__mode=view&_type=entry&blog_id=' .
        $entry->blog_id . '&id=' . $entry->id . '&saved_notify=1');
}

sub start_upload {
    my $app = shift;
    my $perms = $app->{perms} or
        return $app->error("No permissions");
    return $app->error("Permission denied.")
        unless $perms->can_upload;
    my $blog_id = $app->{query}->param('blog_id');
    require MT::Blog;
    my $blog = MT::Blog->load($blog_id);
    $app->build_page('upload.tmpl', {
        local_archive_path => $blog->archive_path,
        local_site_path => $blog->site_path,
    });
}

sub upload_file {
    my $app = shift;
    my $perms = $app->{perms} or
        return $app->error("No permissions");
    return $app->error("Permission denied.")
        unless $perms->can_upload;
    my $q = $app->{query};
    my($fh);
    ## Older versions of CGI.pm didn't have an 'upload' method.
    eval { $fh = $q->upload('file') };
    if ($@ && $@ =~ /^Undefined subroutine/) {
        $fh = $q->param('file');
    }
    return $app->error("You did not choose a file to upload.")
        if !$fh && !$q->param('overwrite') && !$q->param('overwrite');
    my $fname = $q->param('file') || $q->param('fname');
    $fname =~ s!\\!/!g;   ## Change backslashes to forward slashes
    $fname =~ s!^.*/!!;   ## Get rid of full directory paths
    if ($fname =~ m!\0|\|!) {
        return $app->error("Invalid filename '$fname'");
    }
    my $blog_id = $q->param('blog_id');
    require MT::Blog;
    my $blog = MT::Blog->load($blog_id);

    ## Set up the full path to the local file; this path could start
    ## at either the Local Site Path or Local Archive Path, and could
    ## include an extra directory or two in the middle.
    my($base, $extra_path);
    if ($q->param('site_path')) {
        $base = $blog->site_path;
        $extra_path = $q->param('extra_path_site');
    } else {
        $base = $blog->archive_path;
        $extra_path = $q->param('extra_path_archive');
    }
    my $extra_path_save = $extra_path;
    my $path = $base;
    if ($extra_path) {
        if ($extra_path =~ m!\.\.!) {
            return $app->error("Invalid extra path '$extra_path'");
        }
        $path = File::Spec->catdir($path, $extra_path);
        ## Build out the directory structure if it doesn't exist. DirUmask
        ## determines the permissions of the new directories.
        unless (-e $path) {
            require File::Path;
            my $umask = oct $app->{cfg}->DirUmask;
            my $old = umask($umask);
            File::Path::mkpath([$path], 0, 0777);
            umask($old);
        }
    }
    $extra_path = File::Spec->catfile($extra_path, $fname);
    my $local_file = File::Spec->catfile($path, $fname);

    ## Untaint. We have already tested $fname and $extra_path for security
    ## issues above, and we have to assume that we can trust the user's
    ## Local Archive Path setting. So we should be safe.
    ($local_file) = $local_file =~ /(.+)/s;

    ## If $local_file already exists, we try to write the upload to a
    ## tempfile, then ask for confirmation of the upload.
    if (-e $local_file) {
        if (my $ow = $q->param('overwrite')) {
            my $tmp = $q->param('temp');
            if ($tmp =~ m!([^/]+)$!) {
                $tmp = $1;
            } else {
                return $app->error("Invalid temp file name '$tmp'");
            }
            my $tmp_file = File::Spec->catfile($app->{cfg}->TempDir, $tmp);
            if ($ow eq 'yes') {
                $fh = gensym();
                open $fh, $tmp_file
                    or return $app->error("Error opening '$tmp_file': $!");
            } else {
                if (-e $tmp_file) {
                    unlink($tmp_file)
                        or return $app->error("Error deleting '$tmp_file': $!");
                }
                return $app->start_upload;
            }
        } else {
            eval { require File::Temp };
            if ($@) {
                return $app->error("File with name '$fname' already " .
                    "exists. (Install File::Temp if you'd like to be able " .
                    "overwrite existing uploaded files.)");
            }
            my($tmp_fh, $tmp_file) =
                File::Temp::tempfile(DIR => $app->{cfg}->TempDir);
            defined(_write_upload($fh, $tmp_fh))
                or return $app->error("File with name '$fname' already " .
                "exists; Tried to write to tempfile, but open failed: $!");
            (my $tmp = $tmp_file) =~ s!^.*/!!;
            return $app->build_page('upload_confirm.tmpl', {
                temp => $tmp, extra_path => $extra_path_save,
                site_path => scalar $q->param('site_path'),
                fname => $fname });
        }
    }

    ## File does not exist, or else we have confirmed that we can overwrite.
    my $umask = oct $app->{cfg}->UploadUmask;
    my $old = umask($umask);
    defined(my $bytes = _write_upload($fh, $local_file))
        or return $app->error("Error writing upload to '$local_file': $!");
    umask($old);

    ## If we are overwriting the file, that means we still have a temp file
    ## lying around. Delete it.
    if ($q->param('overwrite')) {
        my $tmp = $q->param('temp');
        if ($tmp =~ m!([^/]+)$!) {
            $tmp = $1;
        } else {
            return $app->error("Invalid temp file name '$tmp'");
        }
        my $tmp_file = File::Spec->catfile($app->{cfg}->TempDir, $tmp);
        unlink($tmp_file)
            or return $app->error("Error deleting '$tmp_file': $!");
    }

    ## Use Image::Size to check if the uploaded file is an image, and if so,
    ## record additional image info (width, height).
    eval { require Image::Size; };
    return $app->error("Perl module Image::Size is required to determine " .
                       "width and height of uploaded images.") if $@;
    my($w, $h, $id) = Image::Size::imgsize($local_file);

    my %param = ( width => $w, height => $h, bytes => $bytes,
                  fname => $extra_path,
                  site_path => scalar $q->param('site_path') );
    my $url = $q->param('site_path') ? $blog->site_url : $blog->archive_url;
    $url .= '/' unless $url =~ m!/$!;
    $extra_path =~ s!^/!!;
    $url .= $extra_path;
    $param{url} = $url;
    $param{is_image} = defined($w) && defined($h);
    $app->build_page('upload_complete.tmpl', \%param);
}

sub _write_upload {
    my($upload_fh, $dest_fh) = @_;
    my $fh = gensym();
    if (ref($dest_fh) eq 'GLOB') {
        $fh = $dest_fh;
    } else {
        open $fh, ">$dest_fh" or return;
    }
    binmode $fh;
    binmode $upload_fh;
    my($bytes, $data) = (0);
    while (my $len = read $upload_fh, $data, 8192) {
        print $fh $data;
        $bytes += $len;
    }
    close $fh;
    $bytes;
}

sub start_search_replace {
    my $app = shift;
    my $perms = $app->{perms} or return $app->error("No permissions");
    return $app->error("Permission denied.")
        unless $perms->can_edit_all_posts;
    $app->build_page('search_replace.tmpl',
        { search_cols_title => 1, search_cols_text => 1,
          search_cols_text_more => 1 });
}

sub search_replace {
    my $app = shift;
    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');
    my $perms = $app->{perms} or return $app->error("No permissions");
    return $app->error("Permission denied.")
        unless $perms->can_edit_all_posts;
    my($search, $replace, $do_replace, $case, $is_regex) =
        map scalar $q->param($_), qw( search replace do_replace case is_regex );
    my @cols = $q->param('search_cols');
    unless ($is_regex) {
        $search = quotemeta($search);
        $search = '(?i)' . $search unless $case;
    }
    require MT::Entry;
    my $iter = MT::Entry->load_iter({ blog_id => $blog_id },
        { sort => 'created_on', direction => 'descend' });
    my @data;
    my $i = 1;
    my @to_save;
    while (my $entry = $iter->()) {
        my $match = 0;
        for my $col (@cols) {
            my $text = $entry->$col();
            $text = '' unless defined $text;
            if ($do_replace) {
                $match++ if $text =~ s!$search!$replace!go;
                $entry->$col($text);
            } else {
                $match = $text =~ m!$search!o;
                last if $match;
            }
        }
        if ($match && $do_replace) {
            push @to_save, $entry;
        }
        if ($match) {
            push @data, { entry_title => encode_html($entry->title),
                          entry_id => $entry->id,
                          entry_created_on =>
                              format_ts("%Y.%m.%d", $entry->created_on),
                          is_odd => $i++ % 2 ? 1 : 0,
            };
        }
    }
    for my $entry (@to_save) {
        $entry->save
            or return $app->error("Error saving entry: " . $entry->errstr);
    }
    my %res = (
        have_results => 1,
        results => \@data,
        search => scalar $q->param('search'),
        replace => $replace,
        do_replace => $do_replace,
        case => $case,
        is_regex => $is_regex
    );
    for my $col (@cols) {
        $res{'search_cols_' . $col} = 1;
    }
    $app->build_page('search_replace.tmpl', \%res);
}

1;
