# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: Template.pm,v 1.25 2001/12/13 09:14:31 btrott Exp $

package MT::Template;
use strict;

use vars qw( %TYPES );
%TYPES = (
    'index' => 'Index',
    archive => 'Archive',
    category => 'Category Archive',
    individual => 'Individual',
    comments => 'Comment Listing',
    comment_preview => 'Comment Preview',
    custom => 'Custom',
);

use MT::Object;
@MT::Template::ISA = qw( MT::Object );
__PACKAGE__->install_properties({
    columns => [
        'id', 'blog_id', 'name', 'type', 'outfile', 'text',
        'linked_file', 'linked_file_mtime', 'linked_file_size',
    ],
    indexes => {
        blog_id => 1,
        name => 1,
        type => 1,
    },
    datasource => 'template',
    primary_key => 'id',
});

use MT::Builder;
use MT::Blog;
use MT::ConfigMgr;
use File::Spec;

sub build {
    my $tmpl = shift;
    my($ctx, $cond) = @_;
    my $text = $tmpl->text;
    my $build = MT::Builder->new;
    my $blog_id = $tmpl->blog_id;
    $ctx->stash('blog_id', $blog_id);
    my $blog = MT::Blog->load($blog_id) or
        return $tmpl->error("Load of blog '$blog_id' failed: " .
                            MT::Blog->errstr);
    $ctx->stash('blog', $blog);
    MT::ConfigMgr->instance->TimeOffset($blog->server_offset);
    my $tokens = $build->compile($ctx, $text) or
        return $tmpl->error("Parse error in template '" . $tmpl->name .
                            "': " . $build->errstr);
    defined(my $res = $build->build($ctx, $tokens, $cond)) or
        return $tmpl->error("Build error in template '" . $tmpl->name .
                            "': " . $build->errstr);
    $res;
}

sub save {
    my $tmpl = shift;
    if ($tmpl->linked_file) {
        $tmpl->_sync_to_disk($tmpl->SUPER::text) or return;
    }
    $tmpl->SUPER::save;
}

sub text {
    my $tmpl = shift;
    my $text = $tmpl->SUPER::text(@_);
    unless (@_) {
        if ($tmpl->linked_file) {
            if (my $res = $tmpl->_sync_from_disk) {
                $text = $res;
                $tmpl->SUPER::text($text);
                #$tmpl->SUPER::save;
            }
        }
    }
    $text;
}

sub _sync_from_disk {
    my $tmpl = shift;
    my $lfile = $tmpl->linked_file;
    unless (File::Spec->file_name_is_absolute($lfile)) {
        my $blog = MT::Blog->load($tmpl->blog_id);
        $lfile = File::Spec->catfile($blog->site_path, $lfile);
    }
    return unless -e $lfile;
    my($size, $mtime) = (stat _)[7,9];
    return if $size == $tmpl->linked_file_size &&
              $mtime == $tmpl->linked_file_mtime;
    local *FH;
    open FH, $lfile or return;
    my $c; do { local $/; $c = <FH> };
    close FH;
    $tmpl->linked_file_size($size);
    $tmpl->linked_file_mtime($mtime);
    $c;
}

sub _sync_to_disk {
    my $tmpl = shift;
    my($text) = @_;
    my $lfile = $tmpl->linked_file;
    unless (File::Spec->file_name_is_absolute($lfile)) {
        my $blog = MT::Blog->load($tmpl->blog_id);
        $lfile = File::Spec->catfile($blog->site_path, $lfile);
    }
    local *FH;
    ## If the linked file already exists, and there is no template text
    ## (empty textarea, etc.), then we read the template text from the
    ## linked file, assuming that it should not be overwritten. If the
    ## file does not already exist, or if there is template text, assume
    ## that we should update the linked file.
    if (-e $lfile && !$tmpl->SUPER::text) {
        open FH, $lfile or return;
        do { local $/; $tmpl->SUPER::text(<FH>) };
        close FH;
    } else {
        open FH, ">$lfile" or
            return $tmpl->error("Opening linked file '$lfile' failed: $!");
        print FH $text;
        close FH;
    }
    my($size, $mtime) = (stat $lfile)[7,9];
    $tmpl->linked_file_size($size);
    $tmpl->linked_file_mtime($mtime);
    1;
}

1;
