# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: Util.pm,v 1.31 2001/12/11 00:36:41 btrott Exp $

package MT::Util;
use strict;

use POSIX qw( strftime );
use MT::ConfigMgr;
use Exporter;
@MT::Util::ISA = qw( Exporter );
use vars qw( @EXPORT_OK );
@EXPORT_OK = qw( start_end_day start_end_week start_end_month 
                 html_text_transform encode_html munge_comment
                 offset_time offset_time_list first_n_words
                 archive_file_for format_ts dirify );

sub wday_from_ts {
    my @gm_args = @_;
    require Time::Local;
    my $gm;
    ## If timegm_nocheck is supported, we will use that to determine the
    ## timegm and skip the validity checks. If it is not supported, we just
    ## fall back to using standard timegm, but wrap in an eval to catch
    ## any errors.
    eval { $gm = Time::Local::timegm_nocheck(@gm_args) };
    if ($@) {
        eval { $gm = Time::Local::timegm(@gm_args) };
    }
    my $wday = (gmtime $gm)[6];
    $wday;
}

use POSIX qw( strftime );
sub format_ts {
    my($format, $ts) = @_;
    my @ts = unpack 'A4A2A2A2A2A2', $ts;
    my $wday = wday_from_ts(@ts[5,4,3,2], $ts[1]-1, $ts[0]-1900);
    ## Need to evaluate strftime in scalar context inside format_ts, it
    ## seems; not doing so causes a very strange problem on some machines.
    $ts = strftime $format, @ts[5,4,3,2], $ts[1]-1, $ts[0]-1900, $wday;
    return $ts;
}

{
    my @Days_In = ( -1, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
    sub days_in {
        my($m, $y) = @_;
        return $Days_In[$m] unless $m == 2;
        return $y % 4 == 0 && ($y % 100 != 0 || $y % 400 == 0) ?
            29 : 28;
    }
}

sub start_end_day {
    my $day = substr $_[0], 0, 8;
    return $day . '000000' unless wantarray;
    ($day . "000000", $day . "235959");
}

sub start_end_week {
    my($ts) = @_;
    my($y, $mo, $d, $h, $m, $s) = unpack 'A4A2A2A2A2A2', $ts;
    my $wday = wday_from_ts($s, $m, $h, $d, $mo-1, $y-1900);
    my($sd, $sm, $sy) = ($d - $wday, $mo, $y);
    if ($sd < 1) {
        $sm--;
        $sm = 12, $sy-- if $sm < 1;
        $sd += days_in($sm, $sy);
    }
    my $start = sprintf "%04d%02d%02d%s", $sy, $sm, $sd, "000000";
    return $start unless wantarray;
    my($ed, $em, $ey) = ($d + 6 - $wday, $mo, $y);
    if ($ed > days_in($em, $ey)) {
        $ed -= days_in($em, $ey);
        $em++;
        $em = 1, $ey++ if $em > 12;
    }
    my $end = sprintf "%04d%02d%02d%s", $ey, $em, $ed, "235959";
    ($start, $end);
}

sub start_end_month {
    my($ts) = @_;
    my($y, $mo) = unpack 'A4A2', $ts;
    my $start = sprintf "%04d%02d01000000", $y, $mo;
    return $start unless wantarray;
    my $end = sprintf "%04d%02d%02d235959", $y, $mo, days_in($mo, $y);
    ($start, $end);
}

sub offset_time_list { gmtime offset_time(@_) }

sub offset_time {
    my($ts, $blog, $dir) = @_;
    my $offset;
    if (defined $blog) {
        if (!ref($blog)) {
            require MT::Blog;
            $blog = MT::Blog->load($blog);
        }
        $offset = $blog && $blog->server_offset ? $blog->server_offset : 0;
    } else {
        $offset = MT::ConfigMgr->instance->TimeOffset;
    }
    $offset += 1 if (localtime $ts)[8];
    $offset *= -1 if $dir && $dir eq '-';
    $ts += $offset * 3600;
    $ts;
}

sub html_text_transform {
    my $str = shift;
    $str ||= '';
    my @paras = split /\r?\n\r?\n/, $str;
    for my $p (@paras) {
        $p =~ s!\r?\n!<br />\n!g;
        $p = "<p>$p</p>";
    }
    join "\n\n", @paras;
}

## Should use HTML::Entities for this, but don't want another
## prereq.
sub encode_html {
    my($html) = @_;
    $html = '' unless defined $html;
    $html =~ s!&!&amp;!g;
    $html =~ s!"!&quot;!g;
    $html =~ s!<!&lt;!g;
    $html =~ s!>!&gt;!g;
    $html;
}

sub dirify {
    my $s = lc $_[0];
    $s =~ s![^\w\s]!!g;
    $s =~ tr! !_!s;
    $s;    
}

sub first_n_words {
    my($text, $n) = @_;
    $text =~ s!<[^>]+>!!g;   ## Strip HTML.
    my @words = split /\s+/, $text;
    my $max = @words > $n ? $n : @words;
    return join ' ', @words[0..$max-1];
}

sub munge_comment {
    my($text, $blog) = @_;
    unless ($blog->allow_comment_html) {
        $text =~ s!<[^>]*>!!gs;
        if ($blog->autolink_urls) {
            $text =~ s!(http://\S+)!<a href="$1">$1</a>!g;
        }
    }
    $text;
}

sub archive_file_for {
    my($entry, $blog, $at) = @_;
    return if $at eq 'None';
    my $file;
    my $meth = 'archive_tmpl_' . lc($at);
    my $file_tmpl = $blog->$meth();
    my($ctx);
    if ($file_tmpl) {
        require MT::Template::Context;
        $ctx = MT::Template::Context->new;
    }
    if ($at eq 'Individual') {
        if ($file_tmpl) {
            $ctx->stash('entry', $entry);
            $ctx->{current_timestamp} = $entry->created_on;
        } else {
            $file = sprintf("%06d", $entry->id);
        }
    } elsif ($at eq 'Daily') {
        if ($file_tmpl) {
            $ctx->{current_timestamp} = start_end_day($entry->created_on);
        } else {
            my $start = start_end_day($entry->created_on);
            my($year, $mon, $mday) = unpack 'A4A2A2', $start;
            $file = sprintf("%04d_%02d_%02d", $year, $mon, $mday);
        }
    } elsif ($at eq 'Weekly') {
        if ($file_tmpl) {
            $ctx->{current_timestamp} = start_end_week($entry->created_on);
        } else {
            my $start = start_end_week($entry->created_on);
            my($year, $mon, $mday) = unpack 'A4A2A2', $start;
            $file = sprintf("week_%04d_%02d_%02d", $year, $mon, $mday);
        }
    } elsif ($at eq 'Monthly') {
        if ($file_tmpl) {
            $ctx->{current_timestamp} = start_end_month($entry->created_on);
        } else {
            my $start = start_end_month($entry->created_on);
            my($year, $mon) = unpack 'A4A2', $start;
            $file = sprintf("%04d_%02d", $year, $mon);
        }
    } elsif ($at eq 'Category') {
        if ($file_tmpl) {
            $ctx->stash('archive_category', $entry->category);
        } else {
            my $cat = $entry->category;
            my $label = '';
            if ($cat) {
                $label = dirify($cat->label);
            }
            $file = sprintf("cat_%s", $label);
        }
    } else {
        return $entry->error("Invalid Archive Type setting '$at'");
    }
    if ($file_tmpl) {
        require MT::Builder;
        my $build = MT::Builder->new;
        my $tokens = $build->compile($ctx, $file_tmpl) or return;
        defined($file = $build->build($ctx, $tokens)) or return;
    } else {
        my $ext = $blog->file_extension || 'html';
        $file .= '.' . $ext;
    }
    $file;
}

1;
